@@ -3,10 +3,10 @@
  *  Module    : extern.h
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2011-10-27
+ *  Updated   : 2012-02-20
  *  Notes     :
  *
- * Copyright (c) 1997-2011 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1997-2012 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -66,6 +66,15 @@
 #ifdef DECL_ATOL
 	extern long atol(const char *);
 #endif /* DECL_ATOL */
+#ifdef HAVE_ATOLL
+#	ifdef DECL_ATOLL
+		extern long long atoll(const char *);
+#	endif /* DECL_ATOLL */
+#else
+#	if defined(HAVE_ATOQ) && defined(DECL_ATOQ)
+		 extern long long atoq(const char *);
+#	endif /* HAVE_ATOQ && DECL_ATOQ */
+#endif /* HAVE_ATOLL */
 #ifndef HAVE_MEMCPY
 #	ifdef DECL_BCOPY
 		extern int bcopy(char *, char *, int);
@@ -147,6 +156,13 @@
 #ifdef DECL_GETWD
 	extern char *getwd(char *);
 #endif /* DECL_GETWD */
+
+#if 0 /* doesn't match prototype in proto.h */
+#	ifdef DECL_HEAPSORT
+	extern int heapsort(void *, size_t, size_t, int (*)(t_comptype*, t_comptype*));
+#	endif /* DECL_HEAPSORT */
+#endif /* 0 */
+
 #ifdef DECL_INET_ADDR
 	extern unsigned long inet_addr(const char *);
 #endif /* DECL_INET_ADDR */
@@ -243,6 +259,9 @@
 #ifdef DECL_STRTOL
 	extern long strtol(const char *, char **, int);
 #endif /* DECL_STRTOL */
+#ifdef DECL_STRTOLL
+	extern long long strtoll(const char *, char **, int);
+#endif /* DECL_STRTOLL */
 #ifdef DECL_SYSTEM
 	extern int system(const char *);
 #endif /* DECL_SYSTEM */
@@ -302,9 +321,9 @@
 	extern int vsnprintf(char *, size_t, const char *, va_list);
 #endif /* DECL_VSNPRINTF */
 #if 0 /* some (most?) systems have "int vsprintf(char *, const char *, va_list)" */
-#ifdef DECL_VSPRINTF
+#	ifdef DECL_VSPRINTF
 	extern int vsprintf(char *, char *, va_list);
-#endif /* DECL_VSPRINTF */
+#	endif /* DECL_VSPRINTF */
 #endif /* 0 */
 
 #ifdef __CYGWIN__
@@ -403,6 +422,7 @@ extern constext *txt_attrs[];
 extern constext *txt_auto_cc_bcc_options[];
 #ifdef HAVE_COLOR
 	extern constext *txt_colors[];
+	extern constext *txt_backcolors[];
 #endif /* HAVE_COLOR */
 extern constext *txt_confirm_choices[];
 extern constext *txt_goto_next_unread_options[];
@@ -1380,6 +1400,7 @@ extern int iso2asc_supported;
 extern int last_resp;
 extern int max_active;
 extern int max_art;
+extern int max_base;
 extern int max_newnews;
 extern int max_save;
 extern int max_scope;
@@ -1400,7 +1421,7 @@ extern int xcol;
 extern int xmouse;
 extern int xrow;
 
-extern long *base;
+extern t_artnum *base;
 
 extern signed long int read_newsrc_lines;
 
@@ -1408,6 +1429,12 @@ extern size_t tabwidth;
 
 extern pid_t process_id;
 
+#ifdef USE_HEAPSORT
+	extern int tin_sort(void *, size_t, size_t, t_compfunc);
+	extern constext *txt_sort_functions[];
+	extern struct opttxt txt_sort_function;
+#endif /* USE_HEAPSORT */
+
 extern struct regex_cache strip_re_regex;
 extern struct regex_cache strip_was_regex;
 extern struct regex_cache uubegin_regex;
@@ -1717,6 +1744,9 @@ extern struct opttxt txt_metamail_prog;
 extern struct opttxt txt_news_headers_to_display;
 extern struct opttxt txt_news_headers_to_not_display;
 extern struct opttxt txt_news_quote_format;
+#if defined(HAVE_ALARM) && defined(SIGALRM)
+	extern struct opttxt txt_nntp_read_timeout_secs;
+#endif /* HAVE_ALARM && SIGALRM */
 extern struct opttxt txt_organization;
 extern struct opttxt txt_pos_first_unread;
 extern struct opttxt txt_post_8bit_header;
