@@ -3,10 +3,10 @@
  *  Module    : tin.h
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2011-08-07
+ *  Updated   : 2012-06-20
  *  Notes     : #include files, #defines & struct's
  *
- * Copyright (c) 1997-2011 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1997-2012 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,10 +44,12 @@
 
 #ifdef HAVE_CONFIG_H
 #	ifndef TIN_AUTOCONF_H
-#		include	<autoconf.h>	/* FIXME: normally we use 'config.h' */
+#		include <autoconf.h>	/* FIXME: normally we use 'config.h' */
 #	endif /* !TIN_AUTOCONF_H */
 #else
-#	error "configure run missing"
+#	ifndef HAVE_CONFDEFS_H
+#		error "configure run missing"
+#	endif /* !HAVE_CONFDEFS_H */
 #endif /* HAVE_CONFIG_H */
 
 
@@ -95,7 +97,7 @@
 #	define SMALL_MEMORY_MACHINE
 #endif /* __amiga__ || __amiga */
 
-#include	<signal.h>
+#include <signal.h>
 
 enum context { cMain, cArt, cAttachment, cAttrib, cConfig, cFilter, cGroup, cHelp, cInfopager, cPage, cPost, cPostCancel, cPostFup, cReconnect, cScope, cSelect, cThread, cURL };
 enum icontext { cNone, cGetline, cPromptCONT, cPromptSLK, cPromptYN };
@@ -104,10 +106,10 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 
 #include <stdio.h>
 #ifdef HAVE_ERRNO_H
-#	include	<errno.h>
+#	include <errno.h>
 #else
 #	ifdef HAVE_SYS_ERRNO_H
-#		include	<sys/errno.h>
+#		include <sys/errno.h>
 /*
 	#	else
 	#		error "No errno.h or sys/errno.h found"
@@ -162,7 +164,19 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	include <sys/param.h>
 #endif /* HAVE_SYS_PARAM_H */
 
-#include	<ctype.h>
+#include <ctype.h>
+
+/*
+ * FIXME: make this autoconf
+ * get around broken tolower()/toupper() macros on
+ * ancient BSDs (at least on sony news os)
+ */
+#if defined(__bsd43__) && defined(__sony_news__)
+#	undef tolower
+#	undef toupper
+#	define tolower(c) ((isalpha(c) && isupper(c)) ? ((unsigned char) (c - 'A' + 'a')) : c)
+#	define toupper(c) ((isalpha(c) && islower(c)) ? ((unsigned char) (c - 'a' + 'A')) : c)
+#endif /* __bsd43__ && __sony_news__ */
 
 #ifdef HAVE_STDLIB_H
 #	include <stdlib.h>
@@ -174,6 +188,57 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	include <getopt.h>
 #endif /* HAVE_GETOPT_H */
 
+
+#if defined(ENABLE_LONG_ARTICLE_NUMBERS) && !defined(SMALL_MEMORY_MACHINE)
+/*
+ * defines and typedefs for 64 bit article numbers
+ *
+ * TODO: what if !CPP_DOES_CONCAT
+ *       add configure check for PRIdLEAST64
+ *       add configure check for SCNdLEAST64
+ */
+#	if defined(HAVE_INT_LEAST64_T) && !defined(HAVE_INTTYPES_H)
+#		undef HAVE_INT_LEAST64_T
+#	endif /* HAVE_INT_LEAST64_T && !HAVE_INTTYPES_H */
+#	if defined(HAVE_INT_LEAST64_T) || defined(HAVE_LONG_LONG) || defined(quad_t)
+#		if defined(HAVE_ATOLL) || defined(HAVE_ATOQ)
+#			ifdef HAVE_STRTOLL
+#				define USE_LONG_ARTICLE_NUMBERS 1
+#			endif /* HAVE_STRTOLL */
+#		endif /* HAVE_ATOLL || HAVE_ATOQ */
+#	endif /* HAVE_INT_LEAST64_T || HAVE_LONG_LONG || quad_t */
+#	ifdef HAVE_STDINT_H
+#		include <stdint.h>
+#	endif	/* HAVE_STDINT_H */
+#endif /* ENABLE_LONG_ARTICLE_NUMBERS && !SMALL_MEMORY_MACHINE */
+#ifdef USE_LONG_ARTICLE_NUMBERS
+#	if defined(HAVE_INT_LEAST64_T) && defined(HAVE_INT64_C)
+#		include <inttypes.h>
+		typedef int_least64_t t_artnum;
+#		define T_ARTNUM_PFMT PRIdLEAST64
+#		define T_ARTNUM_SFMT SCNdLEAST64
+#		define T_ARTNUM_CONST(v) INT64_C(v)
+#	else
+		typedef long long t_artnum;
+#		define T_ARTNUM_PFMT "lld"
+#		define T_ARTNUM_SFMT "lld"
+#		define T_ARTNUM_CONST(v) v ## LL
+#	endif /* HAVE_INT_LEAST64_T && HAVE_INT64_C */
+#	ifdef HAVE_ATOLL
+#		define atoartnum atoll
+#	else
+#		define atoartnum atoq
+#	endif /* HAVE_ATOLL */
+#	define strtoartnum strtoll
+#else
+	typedef long t_artnum;
+#	define T_ARTNUM_PFMT "ld"
+#	define T_ARTNUM_SFMT "ld"
+#	define T_ARTNUM_CONST(v) v ## L
+#	define atoartnum atol
+#	define strtoartnum strtol
+#endif /* USE_LONG_ARTICLE_NUMBERS */
+
 /*
  * FIXME: make this autoconf
  */
@@ -290,7 +355,10 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
  */
 #ifdef HAVE_DIRENT_H
 #	include <dirent.h>
+/* #	define NAMLEN(dirent) strlen((dirent)->d_name) */
 #else
+#	define dirent direct
+/* #	define NAMLEN(dirent) (dirent)->d_namlen */
 #	ifdef HAVE_SYS_NDIR_H
 #		include <sys/ndir.h>
 #	endif /* HAVE_SYS_NDIR_H */
@@ -303,7 +371,7 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #endif /* HAVE_DIRENT_H */
 
 #ifndef DIR_BUF
-#	define DIR_BUF	struct dirent
+#	define DIR_BUF struct dirent
 #endif /* !DIR_BUF */
 
 #ifndef HAVE_UNLINK
@@ -513,11 +581,11 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #ifdef HAVE_LONG_FILE_NAMES
 #	define PATH_PART	"part"
 #	define PATH_PATCH	"patch"
-#	define INDEX_LOCK	"%stin.%s.LCK"
+#	define INDEX_LOCK	"tin.%s.LCK"
 #else
 #	define PATH_PART	""
 #	define PATH_PATCH	"p"
-#	define INDEX_LOCK	"%s%s.LCK"
+#	define INDEX_LOCK	"%s.LCK"
 #endif /* HAVE_LONG_FILE_NAMES */
 
 /*
@@ -690,7 +758,7 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #endif /* BOOL_H */
 
 /* Philip Hazel's Perl regular expressions library */
-#include	<pcre.h>
+#include <pcre.h>
 
 #ifdef HAVE_ICONV
 #	define CHARSET_CONVERSION 1
@@ -836,7 +904,7 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #endif /* !ART_MARK_READ_SELECTED */
 #ifndef ART_MARK_KILLED
 #	define ART_MARK_KILLED 'K'		/* art has been killed (kill_level >0) */
-#endif /*! ART_MARK_KILLED */
+#endif /* !ART_MARK_KILLED */
 #ifndef ART_MARK_DELETED
 #	define ART_MARK_DELETED	'D'	/* art has been marked for deletion (mailgroup) */
 #endif /* !ART_MARK_DELETED */
@@ -897,7 +965,6 @@ enum rc_state { RC_IGNORE, RC_CHECK, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
  */
 #define TIN_ABORT		1			/* User requested abort or timeout */
 
-#define NUM_CONFIRM_CHOICES	8	/* confirm what? */
 #define TINRC_CONFIRM_ACTION	(tinrc.confirm_choice == 1 || tinrc.confirm_choice == 4 || tinrc.confirm_choice == 5 || tinrc.confirm_choice == 7)
 #define TINRC_CONFIRM_TO_QUIT	(tinrc.confirm_choice == 3 || tinrc.confirm_choice == 4 || tinrc.confirm_choice == 6 || tinrc.confirm_choice == 7)
 #define TINRC_CONFIRM_SELECT	(tinrc.confirm_choice == 2 || tinrc.confirm_choice == 5 || tinrc.confirm_choice == 6 || tinrc.confirm_choice == 7)
@@ -931,16 +998,9 @@ enum {
 	MIME_ENCODING_8BIT = 0,
 	MIME_ENCODING_BASE64,
 	MIME_ENCODING_QP,
-	MIME_ENCODING_7BIT,
-	NUM_MIME_ENCODINGS
+	MIME_ENCODING_7BIT
 };
 
-#ifdef CHARSET_CONVERSION			/* can/should do charset conversion via iconv() */
-#	define NUM_MIME_CHARSETS 27	/* # known 'outgoing' charsets */
-#endif /* CHARSET_CONVERSION */
-
-#define NUM_MAILBOX_FORMATS 3		/* MBOX0, MBOXRD, MMDF */
-
 /*
  * Number of charset-traslation tables (iso2asci)
  */
@@ -1124,8 +1184,7 @@ enum {
 enum {
 	INTERACTIVE_NONE = 0,
 	INTERACTIVE_WITH_HEADERS,
-	INTERACTIVE_WITHOUT_HEADERS,
-	NUM_INTERACTIVE_MAILERS
+	INTERACTIVE_WITHOUT_HEADERS
 };
 
 /*
@@ -1340,33 +1399,53 @@ enum {
  */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 #	ifdef HAVE_LIBICUUC
-#		define HAVE_UNICODE_NORMALIZATION 1
+#		define HAVE_UNICODE_NORMALIZATION 3
 #	else
-#		if defined(HAVE_LIBIDN) && defined(HAVE_STRINGPREP_H)
+#		ifdef HAVE_LIBUNISTRING
 #			define HAVE_UNICODE_NORMALIZATION 2
-#		endif /* HAVE_LIBIDN */
+#		else
+#			if defined(HAVE_LIBIDN) && defined(HAVE_STRINGPREP_H)
+#				define HAVE_UNICODE_NORMALIZATION 1
+#			endif /* HAVE_LIBIDN */
+#		endif /* HAVE_LIBUNISTRING */
 #	endif /* HAVE_LIBICUUC */
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 /*
- * normalization forms
+ * normalization forms, we prefer NFC (see RFC 6532) if possible
  */
 #ifdef HAVE_UNICODE_NORMALIZATION
 enum {
 	NORMALIZE_NONE = 0,
-#	ifdef HAVE_LIBICUUC
+#	if (HAVE_UNICODE_NORMALIZATION >= 2)
 	NORMALIZE_NFKC = 1,
 	NORMALIZE_NFKD = 2,
 	NORMALIZE_NFC = 3,
 	NORMALIZE_NFD = 4
+#		define DEFAULT_NORMALIZE NORMALIZE_NFC
 #	else
-#		ifdef HAVE_LIBIDN
 	NORMALIZE_NFKC = 1
-#		endif /* HAVE_LIBIDN */
-#	endif /* HAVE_LIBICUUC */
+#		define DEFAULT_NORMALIZE NORMALIZE_NFKC
+#	endif /* HAVE_UNICODE_NORMALIZATION >= 2 */
 };
 #endif /* HAVE_UNICODE_NORMALIZATION */
 
+#ifdef HAVE_LIBICUUC
+#	ifdef HAVE_UNICODE_USTRING_H
+#		include <unicode/ustring.h>
+#	endif /* HAVE_UNICODE_USTRING_H */
+#	ifdef HAVE_UNICODE_UNORM_H
+#		include <unicode/unorm.h>
+#	endif /* HAVE_UNICODE_UNORM_H */
+#	ifdef HAVE_UNICODE_UIDNA_H
+#		include <unicode/uidna.h>
+#	endif /* HAVE_UNICODE_UIDNA_H */
+#	ifdef HAVE_UNICODE_UBIDI_H
+#		include <unicode/ubidi.h>
+#	endif /* HAVE_UNICODE_UBIDI_H */
+#endif /* HAVE_LIBICUUC */
+
+
 /*
  * used in checking article header before posting
  */
@@ -1434,7 +1513,7 @@ struct t_msgid {
  *	>=0 points to the previous arts[] (struct t_article)
  */
 struct t_article {
-	long artnum;			/* Article number in spool directory for group */
+	t_artnum artnum;		/* Article number in spool directory for group */
 	char *subject;			/* Subject: line from mail header */
 	char *from;			/* From: line from mail header (address) */
 	char *name;			/* From: line from mail header (full name) */
@@ -1657,24 +1736,18 @@ struct t_scope {
 	unsigned global:1;			/* TRUE for scopes from global_attributes_file */
 };
 
-/*
- * TODO: turn longs to int_least64_t
- */
 /*
  * struct t_newsrc - newsrc related info.
  */
 struct t_newsrc {
 	t_bool present:1;		/* update newsrc? */
-	long num_unread;		/* unread articles in group */
-	long xmax;			/* newsrc max */
-	long xmin;			/* newsrc min */
-	long xbitlen;			/* bitmap length (max-min+1) */
+	t_artnum num_unread;		/* unread articles in group */
+	t_artnum xmax;			/* newsrc max */
+	t_artnum xmin;			/* newsrc min */
+	t_artnum xbitlen;			/* bitmap length (max-min+1) */
 	t_bitmap *xbitmap;	/* bitmap read/unread (max-min+1+7)/8 */
 };
 
-/*
- * TODO: turn longs to int_least64_t
- */
 /*
  * struct t_group - newsgroup info from active file
  */
@@ -1684,9 +1757,9 @@ struct t_group {
 	char *description;	/* text from NEWSLIBDIR/newsgroups file */
 	char *spooldir;		/* groups spool directory */
 	char moderated;		/* state of group moderation */
-	long count;			/* article number count */
-	long xmax;			/* max. article number */
-	long xmin;			/* min. article number */
+	t_artnum count;		/* article number count */
+	t_artnum xmax;		/* max. article number */
+	t_artnum xmin;		/* min. article number */
 	unsigned int type:4;		/* grouptype - newsgroup/mailbox/savebox */
 	t_bool inrange:1;		/* TRUE if group selected via # range command */
 	t_bool read_during_session:1;	/* TRUE if group entered during session */
@@ -1858,7 +1931,6 @@ struct t_option {
 	int var_index;		/* index in corresponding table */
 	int *variable;		/* ptr to variable to change */
 	constext **opt_list;	/* ptr to list entries if OPT_LIST */
-	int opt_count;		/* no. of list entries if OPT_LIST */
 	struct opttxt *txt;	/* ptr to information/help on option */
 };
 
@@ -1906,6 +1978,12 @@ typedef struct _TIMEINFO {
 } TIMEINFO;
 
 
+struct t_tintime {
+	time_t tv_sec; /* seconds */
+	long tv_nsec; /* nanoseconds */
+};
+
+
 /*
  * mailcap fields
  * the x-token field is missing, we would need something like
@@ -1958,9 +2036,17 @@ typedef struct urllist {
 
 /* Define a matching function pointer type */
 typedef int (*t_compfunc)(t_comptype, t_comptype);
+typedef void (*t_sortfunc)(void *, size_t, size_t, t_compfunc);
 
 #define _CDECL
 
+/* set to (void)heapsort or qsort */
+#ifndef USE_HEAPSORT
+#	define tin_sort qsort
+#else
+#	define MAX_SORT_FUNCS 1
+#endif /* !USE_HEAPSORT */
+
 /* Seperator between dir part of path & the filename */
 #define DIRSEP	'/'
 
@@ -2152,18 +2238,18 @@ typedef OUTC_RETTYPE (*OutcPtr) (OUTC_ARGS);
 typedef FILE TCP;
 
 #ifndef EXTERN_H
-#	include	"extern.h"
+#	include "extern.h"
 #endif /* !EXTERN_H */
 #ifndef TINRC_H
-#	include	"tinrc.h"
+#	include "tinrc.h"
 #endif /* !TINRC_H */
 #ifndef NNTPLIB_H
-#	include	"nntplib.h"
+#	include "nntplib.h"
 #endif /* !NNTPLIB_H */
 
 #ifndef __CPROTO__
 #	ifndef PROTO_H
-#		include	"proto.h"
+#		include "proto.h"
 #	endif /* !PROTO_H */
 #endif /* !__CPROTO__ */
 
@@ -2234,7 +2320,7 @@ typedef void (*BodyPtr) (char *, FILE *, int);
  */
 #ifdef USE_SOCKS5
 #	define SOCKS
-#	include	<socks.h>
+#	include <socks.h>
 /* socks.h doesn't define prototypes for use */
 extern size_t read(int, char *, size_t);
 extern int dup(int);
