@@ -6,7 +6,7 @@
  *  Updated   : 2011-09-18
  *  Notes     : Group attribute routines
  *
- * Copyright (c) 1993-2011 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1993-2012 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -261,8 +261,8 @@ set_default_state(
 		found = TRUE; \
 		break; \
 	}
-#define MATCH_LIST(pattern, type, table, tablelen) \
-	if (match_list(line, pattern, table, tablelen, &num)) { \
+#define MATCH_LIST(pattern, type, table) \
+	if (match_list(line, pattern, table, &num)) { \
 		set_attrib(type, scope, line, &num); \
 		found = TRUE; \
 		break; \
@@ -374,7 +374,7 @@ read_attributes_file(
 
 				case 'm':
 					MATCH_BOOLEAN("mail_8bit_header=", OPT_ATTRIB_MAIL_8BIT_HEADER);
-					MATCH_LIST("mail_mime_encoding=", OPT_ATTRIB_MAIL_MIME_ENCODING, txt_mime_encodings, NUM_MIME_ENCODINGS);
+					MATCH_LIST("mail_mime_encoding=", OPT_ATTRIB_MAIL_MIME_ENCODING, txt_mime_encodings);
 					MATCH_STRING("maildir=", OPT_ATTRIB_MAILDIR);
 					MATCH_STRING("mailing_list=", OPT_ATTRIB_MAILING_LIST);
 					MATCH_BOOLEAN("mark_ignore_tags=", OPT_ATTRIB_MARK_IGNORE_TAGS);
@@ -382,7 +382,7 @@ read_attributes_file(
 					MATCH_BOOLEAN("mime_forward=", OPT_ATTRIB_MIME_FORWARD);
 					MATCH_STRING("mime_types_to_save=", OPT_ATTRIB_MIME_TYPES_TO_SAVE);
 #ifdef CHARSET_CONVERSION
-					MATCH_LIST("mm_network_charset=", OPT_ATTRIB_MM_NETWORK_CHARSET, txt_mime_charsets, NUM_MIME_CHARSETS);
+					MATCH_LIST("mm_network_charset=", OPT_ATTRIB_MM_NETWORK_CHARSET, txt_mime_charsets);
 #else
 					SKIP_ITEM("mm_network_charset=");
 #endif /* CHARSET_CONVERSION */
@@ -401,7 +401,7 @@ read_attributes_file(
 				case 'p':
 					MATCH_BOOLEAN("pos_first_unread=", OPT_ATTRIB_POS_FIRST_UNREAD);
 					MATCH_BOOLEAN("post_8bit_header=", OPT_ATTRIB_POST_8BIT_HEADER);
-					MATCH_LIST("post_mime_encoding=", OPT_ATTRIB_POST_MIME_ENCODING, txt_mime_encodings, NUM_MIME_ENCODINGS);
+					MATCH_LIST("post_mime_encoding=", OPT_ATTRIB_POST_MIME_ENCODING, txt_mime_encodings);
 					MATCH_BOOLEAN("post_process_view=", OPT_ATTRIB_POST_PROCESS_VIEW);
 					MATCH_INTEGER("post_process_type=", OPT_ATTRIB_POST_PROCESS_TYPE, POST_PROC_YES);
 #ifndef DISABLE_PRINTING
@@ -995,7 +995,7 @@ assign_attributes_to_groups(
 			 * or a 8bit charset but a !8bit encoding, update encoding if needed
 			 */
 			is_7bit = FALSE;
-			for (j = 0; *txt_mime_7bit_charsets[j]; j++) {
+			for (j = 0; txt_mime_7bit_charsets[j] != NULL; j++) {
 				if (!strcasecmp(txt_mime_charsets[group->attribute->mm_network_charset], txt_mime_7bit_charsets[j])) {
 					is_7bit = TRUE;
 					break;
@@ -1124,7 +1124,7 @@ write_attributes_file(
 	fprintf(fp, _("#  group_catchup_on_exit=ON/OFF\n"));
 	fprintf(fp, _("#  mail_8bit_header=ON/OFF\n"));
 	fprintf(fp, _("#  mail_mime_encoding=supported_encoding"));
-	for (i = 0; i < NUM_MIME_ENCODINGS; i++) {
+	for (i = 0; txt_mime_encodings[i] != NULL; i++) {
 		if (!(i % 5))
 			fprintf(fp, "\n#    ");
 		fprintf(fp, "%s, ", txt_mime_encodings[i]);
@@ -1141,7 +1141,7 @@ write_attributes_file(
 	fprintf(fp, _("#  mime_forward=ON/OFF\n"));
 #ifdef CHARSET_CONVERSION
 	fprintf(fp, _("#  mm_network_charset=supported_charset"));
-	for (i = 0; i < NUM_MIME_CHARSETS; i++) {
+	for (i = 0; txt_mime_charsets[i] != NULL; i++) {
 		if (!(i % 5)) /* start new line */
 			fprintf(fp, "\n#    ");
 		fprintf(fp, "%s, ", txt_mime_charsets[i]);
@@ -1156,7 +1156,7 @@ write_attributes_file(
 	fprintf(fp, _("#  pos_first_unread=ON/OFF\n"));
 	fprintf(fp, _("#  post_8bit_header=ON/OFF\n"));
 	fprintf(fp, _("#  post_mime_encoding=supported_encoding"));
-	for (i = 0; i < NUM_MIME_ENCODINGS; i++) {
+	for (i = 0; txt_mime_encodings[i] != NULL; i++) {
 		if (!(i % 5))
 			fprintf(fp, "\n#    ");
 		fprintf(fp, "%s, ", txt_mime_encodings[i]);
