@@ -3,10 +3,10 @@
  *  Module    : init.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2011-04-17
+ *  Updated   : 2012-06-20
  *  Notes     :
  *
- * Copyright (c) 1991-2011 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2012 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -283,6 +283,9 @@ struct t_config tinrc = {
 	SHOW_FROM_NAME,				/* show_author */
 	SORT_ARTICLES_BY_DATE_ASCEND,		/* sort_article_type */
 	SORT_THREADS_BY_SCORE_DESCEND,		/* sort_threads_type */
+#ifdef USE_HEAPSORT
+	0,				/* sort_function default qsort */
+#endif /* USE_HEAPSORT */
 	BOGUS_SHOW,		/* strip_bogus */
 	THREAD_BOTH,		/* thread_articles */
 	THREAD_PERC_DEFAULT,	/* thread_perc */
@@ -410,7 +413,7 @@ struct t_config tinrc = {
 	TRUE,		/* default_filter_select_global */
 	DEFAULT_DATE_FORMAT,	/* date_format */
 #ifdef HAVE_UNICODE_NORMALIZATION
-	NORMALIZE_NFKC,		/* normalization form */
+	DEFAULT_NORMALIZE,		/* normalization form */
 #endif /* HAVE_UNICODE_NORMALIZATION */
 #if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
 	FALSE,		/* render_bidi */
@@ -448,6 +451,9 @@ struct t_config tinrc = {
 	FILTER_SUBJ_CASE_SENSITIVE,		/* attrib_quick_kill_header */
 	FILTER_SUBJ_CASE_SENSITIVE,		/* attrib_quick_select_header */
 	MIME_ENCODING_QP,		/* attrib_mail_mime_encoding */
+#if defined(HAVE_ALARM) && defined(SIGALRM)
+	120,	/* nntp_read_timeout_secs */
+#endif /* HAVE_ALARM && SIGALRM */
 	MIME_ENCODING_8BIT,		/* attrib_post_mime_encoding */
 	POST_PROC_NO,			/* attrib_post_process_type */
 	SHOW_FROM_NAME,			/* attrib_show_author */
@@ -630,6 +636,7 @@ init_selfinfo(
 	FILE *fp;
 	char *ptr;
 	const char *cptr;
+	char tmp[PATH_LEN];
 	struct stat sb;
 	struct passwd *myentry;
 
@@ -810,7 +817,7 @@ init_selfinfo(
 		strncat(ptr, get_val("MM_CHARSET", MM_CHARSET), space);
 		if ((space -= strlen(ptr)) > 0) {
 			strncat(ptr, "\n", space);
-			match_list(ptr, "mm_network_charset=", txt_mime_charsets, NUM_MIME_CHARSETS, &tinrc.mm_network_charset);
+			match_list(ptr, "mm_network_charset=", txt_mime_charsets, &tinrc.mm_network_charset);
 		}
 		free(ptr);
 	}
@@ -876,7 +883,8 @@ init_selfinfo(
 	joinpath(postponed_articles_file, sizeof(postponed_articles_file), rcdir, POSTPONED_FILE);
 	joinpath(save_active_file, sizeof(save_active_file), rcdir, ACTIVE_SAVE_FILE);
 
-	snprintf(lock_file, sizeof(lock_file), INDEX_LOCK, TMPDIR, userid);
+	snprintf(tmp, sizeof(tmp), INDEX_LOCK, userid);
+	joinpath(lock_file, sizeof(lock_file), TMPDIR, tmp);
 
 #ifdef NNTP_ABLE
 	nntp_tcp_port = (unsigned short) atoi(get_val("NNTPPORT", NNTP_TCP_PORT));
@@ -965,12 +973,12 @@ read_site_config(
 		if (match_string(buf, "mm_charset=", tinrc.mm_charset, sizeof(tinrc.mm_charset)))
 			continue;
 #else
-		if (match_list(buf, "mm_charset=", txt_mime_charsets, NUM_MIME_CHARSETS, &tinrc.mm_network_charset))
+		if (match_list(buf, "mm_charset=", txt_mime_charsets, &tinrc.mm_network_charset))
 			continue;
 #endif /* !CHARSET_CONVERSION */
-		if (match_list(buf, "post_mime_encoding=", txt_mime_encodings, NUM_MIME_ENCODINGS, &tinrc.post_mime_encoding))
+		if (match_list(buf, "post_mime_encoding=", txt_mime_encodings, &tinrc.post_mime_encoding))
 			continue;
-		if (match_list(buf, "mail_mime_encoding=", txt_mime_encodings, NUM_MIME_ENCODINGS, &tinrc.mail_mime_encoding))
+		if (match_list(buf, "mail_mime_encoding=", txt_mime_encodings, &tinrc.mail_mime_encoding))
 			continue;
 		if (match_boolean(buf, "disable_gnksa_domain_check=", &disable_gnksa_domain_check))
 			continue;
