@@ -3,10 +3,10 @@
  *  Module    : select.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2011-11-10
+ *  Updated   : 2012-03-04
  *  Notes     :
  *
- * Copyright (c) 1991-2011 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2012 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -342,7 +342,7 @@ selection_page(
 				break;
 
 			case GLOBAL_OPTION_MENU:
-				config_page(selmenu.max ? CURR_GROUP.name : NULL);
+				config_page(selmenu.max ? CURR_GROUP.name : NULL, signal_context);
 				show_selection_page();
 				break;
 
@@ -639,7 +639,7 @@ build_gline(
 		strcpy(tmp, "    #");
 	else if (active[my_group[i]].newsrc.num_unread) {
 		int getart_limit;
-		long num_unread;
+		t_artnum num_unread;
 
 		getart_limit = cmdline.args & CMDLINE_GETART_LIMIT ? cmdline.getart_limit : tinrc.getart_limit;
 		num_unread = active[my_group[i]].newsrc.num_unread;
@@ -813,7 +813,7 @@ yank_active_file(
 
 
 /*
- * Sort active[] and associated qsort() helper function
+ * Sort active[] and associated tin_sort() helper function
  */
 static int
 active_comp(
@@ -876,7 +876,7 @@ sort_active_file(
 {
 	save_restore_curr_group(TRUE);
 
-	qsort(active, (size_t) num_active, sizeof(struct t_group), active_comp);
+	tin_sort(active, (size_t) num_active, sizeof(struct t_group), active_comp);
 	group_rehash(yanked_out);
 
 	selmenu.curr = save_restore_curr_group(FALSE);
