@@ -3,10 +3,10 @@
  *  Module    : thread.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2011-01-29
+ *  Updated   : 2012-03-04
  *  Notes     :
  *
- * Copyright (c) 1991-2011 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2012 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -387,7 +387,7 @@ thread_page(
 	char key[MAXKEYLEN];
 	char mark[] = { '\0', '\0' };
 	int i, n;
-	long old_artnum = 0L;
+	t_artnum old_artnum = T_ARTNUM_CONST(0);
 	t_bool repeat_search;
 	t_function func;
 
@@ -656,7 +656,7 @@ thread_page(
 			case GLOBAL_OPTION_MENU:
 				n = find_response(thread_basenote, thdmenu.curr);
 				old_artnum = arts[n].artnum;
-				config_page(group->name);
+				config_page(group->name, signal_context);
 				if ((n = find_artnum(old_artnum)) == -1 || which_thread(n) == -1) { /* We have lost the thread */
 					pos_first_unread_thread();
 					ret_code = GRP_EXIT;
